//
//  FrameView.m
//  Rotation
//
//  Created by Clemens Wagner on 20.06.15.
//  Copyright (c) 2015 Clemens Wagner. All rights reserved.
//

#import "FrameView.h"

IB_DESIGNABLE
@interface AlignmentView : UIView

@end

@implementation FrameView

- (void)layoutSubviews {
    [self updateAlignmentMask];
    [super layoutSubviews];
}

- (UIColor *)frameColor {
    return [UIColor colorWithCGColor:self.layer.borderColor];
}

- (void)setFrameColor:(UIColor *)inColor {
    self.layer.borderColor = inColor.CGColor;
}

- (CGFloat)strokeWidth {
    return self.layer.borderWidth;
}

- (void)setStrokeWidth:(CGFloat)inWidth {
    self.layer.borderWidth = inWidth;
}

- (BOOL)pointInside:(CGPoint)inPoint withEvent:(UIEvent *)inEvent {
    CGRect theBounds = self.bounds;
    CGFloat theInset = self.frameInset;
    CGRect theInsetRect = CGRectInset(theBounds, theInset, theInset);

    return CGRectContainsPoint(theBounds, inPoint) && !CGRectContainsPoint(theInsetRect, inPoint);
}

- (void)setNeedsDisplay {
    [super setNeedsDisplay];
    [self updateAlignmentMask];
}

- (void)drawRect:(CGRect)inRect {
    CGFloat theInset = self.frameInset;
    CGFloat theClipInset = theInset + self.strokeWidth / 2.0;
    CGRect theBounds = self.bounds;
    CGRect theInsetRect = CGRectInset(theBounds, theInset, theInset);
    CGRect theClipRect = CGRectInset(theBounds, theClipInset, theClipInset);
    UIBezierPath *thePath = [UIBezierPath bezierPath];
    
    [self.frameBackgoundColor setFill];
    [self.frameColor setStroke];
    UIRectFill(theBounds);
    UIRectFrame(CGRectInset(theBounds, theInset / 3.0, theInset / 3.0));
    UIRectFrame(theInsetRect);
    [thePath moveToPoint:theBounds.origin];
    [thePath addLineToPoint:theInsetRect.origin];
    [thePath moveToPoint:CGPointMake(CGRectGetMinX(theBounds), CGRectGetMaxY(theBounds))];
    [thePath addLineToPoint:CGPointMake(CGRectGetMinX(theBounds) + theInset, CGRectGetMaxY(theBounds) - theInset)];
    [thePath moveToPoint:CGPointMake(CGRectGetMaxX(theBounds), CGRectGetMaxY(theBounds))];
    [thePath addLineToPoint:CGPointMake(CGRectGetMaxX(theBounds) - theInset, CGRectGetMaxY(theBounds) - theInset)];
    [thePath moveToPoint:CGPointMake(CGRectGetMaxX(theBounds), CGRectGetMinY(theBounds))];
    [thePath addLineToPoint:CGPointMake(CGRectGetMaxX(theBounds) - theInset, CGRectGetMinY(theBounds) + theInset)];
    [thePath stroke];
    [[UIColor clearColor] setFill];
    UIRectFillUsingBlendMode(theClipRect, kCGBlendModeDestinationAtop);
}

- (void)updateAlignmentMask {
    CGRect theBounds = self.bounds;
    CGFloat theInset = self.frameInset + self.strokeWidth / 2.0;
    UIView *theView = self.subviews.firstObject;
    UIView *theMaskView = theView.maskView;
    
    if(theMaskView == nil) {
        theMaskView = [UIView new];
        theMaskView.backgroundColor = [UIColor blackColor];
        theView.maskView = theMaskView;
    }
    theMaskView.frame = CGRectInset(theBounds, theInset, theInset);
}

@end

@implementation AlignmentView

@end
